@@ -174,11 +174,11 @@ and keyboardEvent = object
   method metaKey : bool t readonly_prop
 end
 
-and wheelEvent = object (* All browsers but Firefox *)
+and mousewheelEvent = object (* All browsers but Firefox *)
   inherit mouseEvent
-  method delta : int readonly_prop
-  method deltaX : int optdef readonly_prop
-  method deltaY : int optdef readonly_prop
+  method wheelDelta : int readonly_prop
+  method wheelDeltaX : int optdef readonly_prop
+  method wheelDeltaY : int optdef readonly_prop
 end
 
 and mouseScrollEvent = object (* Firefox *)
@@ -281,6 +281,13 @@ and storage = object
   method clear : unit meth
 end
 
+and hashChangeEvent = object
+  inherit event
+  method oldURL : js_string t readonly_prop
+  method newURL : js_string t readonly_prop
+end
+
+
 (** {2 HTML elements} *)
 
 and nodeSelector = object
@@ -346,13 +353,13 @@ end
 
 and clientRectList = object
   method length : int readonly_prop
-  method item : int -> clientRect t optdef meth
+  method item : int -> clientRect t opt meth
 end
 
 (** Collection of HTML elements *)
 class type ['node] collection = object
   method length : int readonly_prop
-  method item : int -> 'node t optdef meth
+  method item : int -> 'node t opt meth
   method namedItem : js_string t -> 'node t opt meth
 end
 
@@ -454,6 +461,7 @@ class type selectElement = object ('self)
   method focus : unit meth
 
   method onchange : ('self t, event t) event_listener prop
+  method oninput : ('self t, event t) event_listener prop
 end
 
 class type inputElement = object ('self)
@@ -484,6 +492,7 @@ class type inputElement = object ('self)
 
   method onselect : ('self t, event t) event_listener prop
   method onchange : ('self t, event t) event_listener prop
+  method oninput : ('self t, event t) event_listener prop
 end
 
 class type textAreaElement = object ('self)
@@ -505,6 +514,7 @@ class type textAreaElement = object ('self)
 
   method onselect : ('self t, event t) event_listener prop
   method onchange : ('self t, event t) event_listener prop
+  method oninput : ('self t, event t) event_listener prop
 end
 
 class type buttonElement = object
@@ -952,6 +962,13 @@ class type navigator = object
   method userLanguage : js_string t optdef readonly_prop
 end
 
+class type screen = object
+  method width : int readonly_prop
+  method height : int readonly_prop
+  method availWidth : int readonly_prop
+  method availHeight : int readonly_prop
+end
+
 type interval_id
 type timeout_id
 
@@ -965,10 +982,12 @@ class type window = object
   method navigator : navigator t
   method getSelection : selection t meth
   method close : unit meth
+  method closed : bool t readonly_prop
   method stop : unit meth
   method focus : unit meth
   method blur : unit meth
   method scroll : int -> int -> unit meth
+  method screen : screen t readonly_prop
 
   method sessionStorage : storage t optdef readonly_prop
   method localStorage : storage t optdef readonly_prop
@@ -996,6 +1015,7 @@ class type window = object
   method onresize : (window t, event t) event_listener prop
   method onscroll : (window t, event t) event_listener prop
   method onpopstate : (window t, popStateEvent t) event_listener prop
+  method onhashchange : (window t, hashChangeEvent t) event_listener prop
 end
 
 val window : window t
@@ -1076,7 +1096,7 @@ module Event : sig
   val keypress : keyboardEvent t typ
   val keydown : keyboardEvent t typ
   val keyup : keyboardEvent t typ
-  val mousewheel : wheelEvent t typ
+  val mousewheel : mousewheelEvent t typ
   val _DOMMouseScroll : mouseScrollEvent t typ
   val touchstart : touchEvent t typ
   val touchmove : touchEvent t typ
@@ -1089,6 +1109,9 @@ module Event : sig
   val dragleave : dragEvent t typ
   val drag : dragEvent t typ
   val drop : dragEvent t typ
+  val hashchange : hashChangeEvent t typ
+  val change : event t typ
+  val input : event t typ
 
   val make : string -> 'a typ
 end
@@ -1295,7 +1318,7 @@ val opt_tagged : #element t opt -> taggedElement option
 type taggedEvent =
   | MouseEvent of mouseEvent t
   | KeyboardEvent of keyboardEvent t
-  | WheelEvent of wheelEvent t
+  | MousewheelEvent of mousewheelEvent t
   | MouseScrollEvent of mouseScrollEvent t
   | PopStateEvent of popStateEvent t
   | OtherEvent of event t
@@ -1376,7 +1399,7 @@ module CoerceTo : sig
 
   val mouseEvent : #event t -> mouseEvent t opt
   val keyboardEvent : #event t -> keyboardEvent t opt
-  val wheelEvent : #event t -> wheelEvent t opt
+  val wheelEvent : #event t -> mousewheelEvent t opt
   val mouseScrollEvent : #event t -> mouseScrollEvent t opt
   val popStateEvent : #event t -> popStateEvent t opt
 
