@@ -30,11 +30,14 @@ module Unsafe = struct
 
   external get : 'a -> 'b -> 'c = "caml_js_get"
   external set : 'a -> 'b -> 'c -> unit = "caml_js_set"
+  external delete : 'a -> 'b -> unit = "caml_js_delete"
   external call : 'a -> 'b -> any array -> 'c = "caml_js_call"
   external fun_call : 'a -> any array -> 'b = "caml_js_fun_call"
   external meth_call : 'a -> string -> any array -> 'b = "caml_js_meth_call"
   external new_obj : 'a -> any array -> 'b = "caml_js_new"
 
+  external obj : (string * any) array -> 'a = "caml_js_object"
+
   external equals : 'a -> 'b -> bool = "caml_js_equals"
 
   external pure_expr : (unit -> 'a) -> 'a = "caml_js_pure_expr"
