@@ -21,14 +21,14 @@
 
 //Provides: caml_weak_create
 function caml_weak_create (n) {
-  var x = [0];
-  x.length = n + 2;
+  var x = [251];
+  x.length = n + 1;
   return x;
 }
 //Provides: caml_weak_set
-function caml_weak_set(x, i, v) { x[i] = v; return 0; }
+function caml_weak_set(x, i, v) { x[i + 1] = v; return 0; }
 //Provides: caml_weak_get mutable
-function caml_weak_get(x, i) { return (x[i]===undefined)?0:x[i]; }
+function caml_weak_get(x, i) { return (x[i + 1]===undefined)?0:x[i + 1]; }
 //Provides: caml_weak_get_copy mutable
 //Requires: caml_weak_get
 function caml_weak_get_copy(x, i) {
@@ -39,9 +39,9 @@ function caml_weak_get_copy(x, i) {
   return y;
 }
 //Provides: caml_weak_check mutable
-function caml_weak_check(x, i) { return x[i]!==undefined && x[i] !==0; }
-//Provides: caml_weak_blit
-function caml_weak_blit(s, i, d, j, l) {
-  for (var k = 0; k < l; k++) d[j + k] = s[i + k];
-  return 0;
+function caml_weak_check(x, i) {
+  return x[i + 1]!==undefined && x[i + 1] !==0;
 }
+//Provides: caml_weak_blit
+//Requires: caml_array_blit
+var caml_weak_blit = caml_array_blit;
